@@ -1,4 +1,5 @@
 #! /usr/bin/perl -w
+# -*- tab-width: 8; indent-tabs-mode: t; cperl-indent-level: 4 -*-
 
 # uscan: This program looks for watchfiles and checks upstream ftp sites
 # for later versions of the software.
@@ -56,6 +57,7 @@ eval { require Crypt::SSLeay; };
 if ($@) {
     $haveSSL = 0;
 }
+my $havegpgv = (-x '/usr/bin/gpgv');
 
 # Did we find any new upstream versions on our wanderings?
 our $found = 0;
@@ -65,8 +67,8 @@ sub process_watchfile ($$$$);
 sub recursive_regex_dir ($$$);
 sub newest_dir ($$$$$);
 sub dehs_msg ($);
-sub dehs_warn ($);
-sub dehs_die ($);
+sub uscan_warn (@);
+sub uscan_die (@);
 sub dehs_output ();
 sub quoted_regex_replace ($);
 sub safe_replace ($$);
@@ -318,36 +320,32 @@ $verbose = $opt_verbose if defined $opt_verbose;
 $dehs = $opt_dehs if defined $opt_dehs;
 $user_agent_string = $opt_user_agent if defined $opt_user_agent;
 $download_version = $opt_download_version if defined $opt_download_version;
-if ($dehs) {
-    $SIG{'__WARN__'} = \&dehs_warn;
-    $SIG{'__DIE__'} = \&dehs_die;
-}
 
 if (defined $opt_level) {
     if ($opt_level =~ /^[012]$/) { $check_dirname_level = $opt_level; }
     else {
-	die "$progname: unrecognised --check-dirname-level value (allowed are 0,1,2)\n";
+	uscan_die "$progname: unrecognised --check-dirname-level value (allowed are 0,1,2)\n";
     }
 }
 
 $check_dirname_regex = $opt_regex if defined $opt_regex;
 
 if (defined $opt_package) {
-    die "$progname: --package requires the use of --watchfile\nas well; run $progname --help for more details\n"
+    uscan_die "$progname: --package requires the use of --watchfile\nas well; run $progname --help for more details\n"
 	unless defined $opt_watchfile;
     $download = -$download unless defined $opt_download;
 }
 
-die "$progname: Can't use --verbose if you're using --dehs!\n"
+uscan_die "$progname: Can't use --verbose if you're using --dehs!\n"
     if $verbose and $dehs;
 
-die "$progname: Can't use --report-status if you're using --verbose!\n"
+uscan_die "$progname: Can't use --report-status if you're using --verbose!\n"
     if $verbose and $report;
 
-die "$progname: Can't use --report-status if you're using --download!\n"
+uscan_die "$progname: Can't use --report-status if you're using --download!\n"
     if $download and $report;
 
-warn "$progname: You're going to get strange (non-XML) output using --debug and --dehs together!\n"
+uscan_warn "$progname: You're going to get strange (non-XML) output using --debug and --dehs together!\n"
     if $debug and $dehs;
 
 # We'd better be verbose if we're debugging
@@ -394,7 +392,7 @@ $user_agent->timeout($timeout);
 $user_agent->agent($user_agent_string);
 
 if (defined $opt_watchfile) {
-    die "Can't have directory arguments if using --watchfile" if @ARGV;
+    uscan_die "Can't have directory arguments if using --watchfile" if @ARGV;
 
     # no directory traversing then, and things are very simple
     if (defined $opt_package) {
@@ -403,23 +401,23 @@ if (defined $opt_watchfile) {
     } else {
 	# Check for debian/changelog file
 	until (-r 'debian/changelog') {
-	    chdir '..' or die "$progname: can't chdir ..: $!\n";
+	    chdir '..' or uscan_die "$progname: can't chdir ..: $!\n";
 	    if (cwd() eq '/') {
-		die "$progname: cannot find readable debian/changelog anywhere!\nAre you in the source code tree?\n";
+		uscan_die "$progname: cannot find readable debian/changelog anywhere!\nAre you in the source code tree?\n";
 	    }
 	}
 
 	# Figure out package info we need
 	my $changelog = `dpkg-parsechangelog`;
 	unless ($? == 0) {
-	    die "$progname: Problems running dpkg-parsechangelog\n";
+	    uscan_die "$progname: Problems running dpkg-parsechangelog\n";
 	}
 
 	my ($package, $debversion, $uversion);
 	$changelog =~ /^Source: (.*?)$/m and $package=$1;
 	$changelog =~ /^Version: (.*?)$/m and $debversion=$1;
 	if (! defined $package || ! defined $debversion) {
-	    die "$progname: Problems determining package name and/or version from\n  debian/changelog\n";
+	    uscan_die "$progname: Problems determining package name and/or version from\n  debian/changelog\n";
 	}
 
 	# Check the directory is properly named for safety
@@ -435,7 +433,7 @@ if (defined $opt_watchfile) {
 	    }
 	}
 	if (! $good_dirname) {
-	    die "$progname: not processing watchfile because this directory does not match the package name\n" .
+	    uscan_die "$progname: not processing watchfile because this directory does not match the package name\n" .
 		"   or the settings of the--check-dirname-level and --check-dirname-regex options if any.\n";
 	}
 
@@ -466,7 +464,7 @@ print "-- Scanning for watchfiles in @ARGV\n" if $verbose;
 # otherwise.
 my @dirs;
 open FIND, '-|', 'find', @ARGV, qw(-follow -type d -name debian -print)
-    or die "$progname: couldn't exec find: $!\n";
+    or uscan_die "$progname: couldn't exec find: $!\n";
 
 while (<FIND>) {
     chomp;
@@ -474,19 +472,19 @@ while (<FIND>) {
 }
 close FIND;
 
-die "$progname: No debian directories found\n" unless @dirs;
+uscan_die "$progname: No debian directories found\n" unless @dirs;
 
 my @debdirs = ();
 
 my $origdir = cwd;
 for my $dir (@dirs) {
     unless (chdir $origdir) {
-	warn "$progname warning: Couldn't chdir back to $origdir, skipping: $!\n";
+	uscan_warn "$progname warning: Couldn't chdir back to $origdir, skipping: $!\n";
 	next;
     }
     $dir =~ s%/debian$%%;
     unless (chdir $dir) {
-	warn "$progname warning: Couldn't chdir $dir, skipping: $!\n";
+	uscan_warn "$progname warning: Couldn't chdir $dir, skipping: $!\n";
 	next;
     }
 
@@ -495,7 +493,7 @@ for my $dir (@dirs) {
 	# Figure out package info we need
 	my $changelog = `dpkg-parsechangelog`;
 	unless ($? == 0) {
-	    warn "$progname warning: Problems running dpkg-parsechangelog in $dir, skipping\n";
+	    uscan_warn "$progname warning: Problems running dpkg-parsechangelog in $dir, skipping\n";
 	    next;
 	}
 
@@ -503,7 +501,7 @@ for my $dir (@dirs) {
 	$changelog =~ /^Source: (.*?)$/m and $package=$1;
 	$changelog =~ /^Version: (.*?)$/m and $debversion=$1;
 	if (! defined $package || ! defined $debversion) {
-	    warn "$progname warning: Problems determining package name and/or version from\n  $dir/debian/changelog, skipping\n";
+	    uscan_warn "$progname warning: Problems determining package name and/or version from\n  $dir/debian/changelog, skipping\n";
 	    next;
 	}
 
@@ -536,23 +534,23 @@ for my $dir (@dirs) {
 	push @debdirs, [$debversion, $dir, $package, $uversion];
     }
     elsif (-r 'debian/watch') {
-	warn "$progname warning: Found watchfile in $dir,\n  but couldn't find/read changelog; skipping\n";
+	uscan_warn "$progname warning: Found watchfile in $dir,\n  but couldn't find/read changelog; skipping\n";
 	next;
     }
     elsif (-f 'debian/watch') {
-	warn "$progname warning: Found watchfile in $dir,\n  but it is not readable; skipping\n";
+	uscan_warn "$progname warning: Found watchfile in $dir,\n  but it is not readable; skipping\n";
 	next;
     }
 }
 
-warn "$progname: no watch file found\n" if (@debdirs == 0 and $report);
+uscan_warn "$progname: no watch file found\n" if (@debdirs == 0 and $report);
 
 # Was there a --uversion option?
 if (defined $opt_uversion) {
     if (@debdirs == 1) {
 	$debdirs[0][3] = $opt_uversion;
     } else {
-	warn "$progname warning: ignoring --uversion as more than one debian/watch file found\n";
+	uscan_warn "$progname warning: ignoring --uversion as more than one debian/watch file found\n";
     }
 }
 
@@ -573,16 +571,16 @@ for my $debdir (@debdirs) {
     my $version = $$debdir[2];
 
     if (exists $donepkgs{$parentdir}{$package}) {
-	warn "$progname warning: Skipping $dir/debian/watch\n  as this package has already been scanned successfully\n";
+	uscan_warn "$progname warning: Skipping $dir/debian/watch\n  as this package has already been scanned successfully\n";
 	next;
     }
 
     unless (chdir $origdir) {
-	warn "$progname warning: Couldn't chdir back to $origdir, skipping: $!\n";
+	uscan_warn "$progname warning: Couldn't chdir back to $origdir, skipping: $!\n";
 	next;
     }
     unless (chdir $dir) {
-	warn "$progname warning: Couldn't chdir $dir, skipping: $!\n";
+	uscan_warn "$progname warning: Couldn't chdir $dir, skipping: $!\n";
 	next;
     }
 
@@ -696,13 +694,14 @@ sub process_watchline ($$$$$$)
     # Comma-separated list of features that sites being queried might
     # want to be aware of
     $headers->header('X-uscan-features' => 'enhanced-matching');
+    $headers->header('Accept' => '*/*');
     %dehs_tags = ('package' => $pkg);
 
     if ($watch_version == 1) {
 	($site, $dir, $filepattern, $lastversion, $action) = split ' ', $line, 5;
 
 	if (! defined $lastversion or $site =~ /\(.*\)/ or $dir =~ /\(.*\)/) {
-	    warn "$progname warning: there appears to be a version 2 format line in\n  the version 1 watchfile $watchfile;\n  Have you forgotten a 'version=2' line at the start, perhaps?\n  Skipping the line: $line\n";
+	    uscan_warn "$progname warning: there appears to be a version 2 format line in\n  the version 1 watchfile $watchfile;\n  Have you forgotten a 'version=2' line at the start, perhaps?\n  Skipping the line: $line\n";
 	    return 1;
 	}
 	if ($site !~ m%\w+://%) {
@@ -717,7 +716,7 @@ sub process_watchline ($$$$$$)
 		$filepattern =~ s/\?/./g;
 		$filepattern =~ s/\*/.*/g;
 		$style='old';
-		warn "$progname warning: Using very old style of filename pattern in $watchfile\n  (this might lead to incorrect results): $3\n";
+		uscan_warn "$progname warning: Using very old style of filename pattern in $watchfile\n  (this might lead to incorrect results): $3\n";
 	    }
 	}
 
@@ -736,7 +735,7 @@ sub process_watchline ($$$$$$)
 	    } elsif ($line =~ s/^(\S+)\s+//) {
 		$opts=$1;
 	    } else {
-		warn "$progname warning: malformed opts=... in watchfile, skipping line:\n$origline\n";
+		uscan_warn "$progname warning: malformed opts=... in watchfile, skipping line:\n$origline\n";
 		return 1;
 	    }
 
@@ -765,8 +764,11 @@ sub process_watchline ($$$$$$)
 		elsif ($opt =~ /^downloadurlmangle\s*=\s*(.+)/) {
 		    @{$options{'downloadurlmangle'}} = split /;/, $1;
 		}
+		elsif ($opt =~ /^pgpsigurlmangle\s*=\s*(.+)/) {
+		    @{$options{'pgpsigurlmangle'}} = split /;/, $1;
+		}
 		else {
-		    warn "$progname warning: unrecognised option $opt\n";
+		    uscan_warn "$progname warning: unrecognised option $opt\n";
 		}
 	    }
 	}
@@ -782,19 +784,30 @@ sub process_watchline ($$$$$$)
 	}
 
 	if ((!$lastversion or $lastversion eq 'debian') and not defined $pkg_version) {
-	    warn "$progname warning: Unable to determine current version\n  in $watchfile, skipping:\n  $line\n";
+	    uscan_warn "$progname warning: Unable to determine current version\n  in $watchfile, skipping:\n  $line\n";
 	    return 1;
 	}
 
 	# Check all's OK
 	if (not $filepattern or $filepattern !~ /\(.*\)/) {
-	    warn "$progname warning: Filename pattern missing version delimiters ()\n  in $watchfile, skipping:\n  $line\n";
+	    uscan_warn "$progname warning: Filename pattern missing version delimiters ()\n  in $watchfile, skipping:\n  $line\n";
 	    return 1;
 	}
 
 	# Check validity of options
 	if ($base =~ /^ftp:/ and exists $options{'downloadurlmangle'}) {
-	    warn "$progname warning: downloadurlmangle option invalid for ftp sites,\n  ignoring in $watchfile:\n  $line\n";
+	    uscan_warn "$progname warning: downloadurlmangle option invalid for ftp sites,\n  ignoring in $watchfile:\n  $line\n";
+	}
+
+	# Check validity of options
+	if (exists $options{'pgpsigurlmangle'}) {
+	    if (not (-r 'debian/upstream-signing-key.pgp')) {
+		uscan_warn "$progname warning: pgpsigurlmangle option exists, but debian/upstream-signing-key.pgp does not exist,\n  ignoring in $watchfile:\n  $line\n";
+		delete $options{'pgpsigurlmangle'};
+	    } elsif (! $havegpgv) {
+		uscan_warn "$progname warning: pgpsignurlmangle option exists, but you must have gpgv installed to verify\n  in $watchfile, skipping:\n  $line\n";
+		return 1;
+	    }
 	}
 
 	# Handle sf.net addresses specially
@@ -805,7 +818,7 @@ sub process_watchline ($$$$$$)
 	if ($base =~ m%^(\w+://[^/]+)%) {
 	    $site = $1;
 	} else {
-	    warn "$progname warning: Can't determine protocol and site in\n  $watchfile, skipping:\n  $line\n";
+	    uscan_warn "$progname warning: Can't determine protocol and site in\n  $watchfile, skipping:\n  $line\n";
 	    return 1;
 	}
 
@@ -825,7 +838,7 @@ sub process_watchline ($$$$$$)
 	if (defined $pkg_version) {
 	    $lastversion=$pkg_version;
 	} else {
-	    warn "$progname warning: Unable to determine current version\n  in $watchfile, skipping:\n  $line\n";
+	    uscan_warn "$progname warning: Unable to determine current version\n  in $watchfile, skipping:\n  $line\n";
 	    return 1;
 	}
     }
@@ -834,7 +847,7 @@ sub process_watchline ($$$$$$)
     $mangled_lastversion = $lastversion;
     foreach my $pat (@{$options{'dversionmangle'}}) {
 	if (! safe_replace(\$mangled_lastversion, $pat)) {
-	    warn "$progname: In $watchfile, potentially"
+	    uscan_warn "$progname: In $watchfile, potentially"
 	      . " unsafe or malformed dversionmangle"
 	      . " pattern:\n  '$pat'"
 	      . " found. Skipping watchline\n"
@@ -849,7 +862,7 @@ sub process_watchline ($$$$$$)
 
     # Check all's OK
     if ($pattern !~ /\(.*\)/) {
-	warn "$progname warning: Filename pattern missing version delimiters ()\n  in $watchfile, skipping:\n  $line\n";
+	uscan_warn "$progname warning: Filename pattern missing version delimiters ()\n  in $watchfile, skipping:\n  $line\n";
 	return 1;
     }
 
@@ -859,16 +872,16 @@ sub process_watchline ($$$$$$)
 
     # What is the most recent file, based on the filenames?
     # We first have to find the candidates, then we sort them using
-    # Devscripts::Versort::versort
+    # Devscripts::Versort::upstream_versort
     if ($site =~ m%^http(s)?://%) {
 	if (defined($1) and !$haveSSL) {
-	    die "$progname: you must have the libcrypt-ssleay-perl package installed\nto use https URLs\n";
+	    uscan_die "$progname: you must have the libcrypt-ssleay-perl package installed\nto use https URLs\n";
 	}
 	print STDERR "$progname debug: requesting URL $base\n" if $debug;
 	$request = HTTP::Request->new('GET', $base, $headers);
 	$response = $user_agent->request($request);
 	if (! $response->is_success) {
-	    warn "$progname warning: In watchfile $watchfile, reading webpage\n  $base failed: " . $response->status_line . "\n";
+	    uscan_warn "$progname warning: In watchfile $watchfile, reading webpage\n  $base failed: " . $response->status_line . "\n";
 	    return 1;
 	}
 
@@ -947,7 +960,7 @@ sub process_watchline ($$$$$$)
 			 	$href =~ m&^$_pattern$&);
 			foreach my $pat (@{$options{'uversionmangle'}}) {
 			    if (! safe_replace(\$mangled_version, $pat)) {
-				warn "$progname: In $watchfile, potentially"
+				uscan_warn "$progname: In $watchfile, potentially"
 			 	 . " unsafe or malformed uversionmangle"
 				  . " pattern:\n  '$pat'"
 				  . " found. Skipping watchline\n"
@@ -970,23 +983,23 @@ sub process_watchline ($$$$$$)
 		if (@vhrefs) {
 		    ($newversion, $newfile) = @{$vhrefs[0]};
 		} else {
-		    warn "$progname warning: In $watchfile no matching hrefs for version $download_version"
+		    uscan_warn "$progname warning: In $watchfile no matching hrefs for version $download_version"
 			. " in watch line\n  $line\n";
 		    return 1;
 		}
 	    } else {
-		@hrefs = Devscripts::Versort::versort(@hrefs);
+		@hrefs = Devscripts::Versort::upstream_versort(@hrefs);
 		($newversion, $newfile) = @{$hrefs[0]};
 	    }
 	} else {
-	    warn "$progname warning: In $watchfile,\n  no matching hrefs for watch line\n  $line\n";
+	    uscan_warn "$progname warning: In $watchfile,\n  no matching hrefs for watch line\n  $line\n";
 	    return 1;
 	}
     }
     else {
 	# Better be an FTP site
 	if ($site !~ m%^ftp://%) {
-	    warn "$progname warning: Unknown protocol in $watchfile, skipping:\n  $site\n";
+	    uscan_warn "$progname warning: Unknown protocol in $watchfile, skipping:\n  $site\n";
 	    return 1;
 	}
 
@@ -1001,7 +1014,7 @@ sub process_watchline ($$$$$$)
 	    else { delete $ENV{'FTP_PASSIVE'}; }
 	}
 	if (! $response->is_success) {
-	    warn "$progname warning: In watchfile $watchfile, reading FTP directory\n  $base failed: " . $response->status_line . "\n";
+	    uscan_warn "$progname warning: In watchfile $watchfile, reading FTP directory\n  $base failed: " . $response->status_line . "\n";
 	    return 1;
 	}
 
@@ -1025,7 +1038,7 @@ sub process_watchline ($$$$$$)
 		my $mangled_version = join(".", $file =~ m/^$pattern$/);
 		foreach my $pat (@{$options{'uversionmangle'}}) {
 		    if (! safe_replace(\$mangled_version, $pat)) {
-			warn "$progname: In $watchfile, potentially"
+			uscan_warn "$progname: In $watchfile, potentially"
 			  . " unsafe or malformed uversionmangle"
 			  . " pattern:\n  '$pat'"
 			  . " found. Skipping watchline\n"
@@ -1044,7 +1057,7 @@ sub process_watchline ($$$$$$)
 		    my $mangled_version = join(".", $file =~ m/^$filepattern$/);
 		    foreach my $pat (@{$options{'uversionmangle'}}) {
 			if (! safe_replace(\$mangled_version, $pat)) {
-			    warn "$progname: In $watchfile, potentially"
+			    uscan_warn "$progname: In $watchfile, potentially"
 			      . " unsafe or malformed uversionmangle"
 			      . " pattern:\n  '$pat'"
 			      . " found. Skipping watchline\n"
@@ -1067,16 +1080,16 @@ sub process_watchline ($$$$$$)
 		if (@vfiles) {
 		    ($newversion, $newfile) = @{$vfiles[0]};
 		} else {
-		    warn "$progname warning: In $watchfile no matching files for version $download_version"
+		    uscan_warn "$progname warning: In $watchfile no matching files for version $download_version"
 			. " in watch line\n  $line\n";
 		    return 1;
 		}
 	    } else {
-		@files = Devscripts::Versort::versort(@files);
+		@files = Devscripts::Versort::upstream_versort(@files);
 		($newversion, $newfile) = @{$files[0]};
 	    }
 	} else {
-	    warn "$progname warning: In $watchfile no matching files for watch line\n  $line\n";
+	    uscan_warn "$progname warning: In $watchfile no matching files for watch line\n  $line\n";
 	    return 1;
 	}
     }
@@ -1091,7 +1104,7 @@ sub process_watchline ($$$$$$)
 	if ($newversion =~ /^\D*(\d+\.(?:\d+\.)*\d+)\D*$/) {
 	    $newversion = $1;
 	} else {
-	    warn <<"EOF";
+	    uscan_warn <<"EOF";
 $progname warning: In $watchfile, couldn\'t determine a
   pure numeric version number from the file name for watch line
   $line
@@ -1108,7 +1121,7 @@ EOF
     }
     foreach my $pat (@{$options{'filenamemangle'}}) {
 	if (! safe_replace(\$newfile_base, $pat)) {
-	    warn "$progname: In $watchfile, potentially"
+	    uscan_warn "$progname: In $watchfile, potentially"
 	      . " unsafe or malformed filenamemangle"
 	      . " pattern:\n  '$pat'"
 	      . " found. Skipping watchline\n"
@@ -1127,6 +1140,7 @@ EOF
 
     # So what have we got to report now?
     my $upstream_url;
+    my $pgpsig_url;
     # Upstream URL?  Copying code from below - ugh.
     if ($site =~ m%^https?://%) {
 	# absolute URL?
@@ -1151,7 +1165,7 @@ EOF
 		}
 		if (!defined($upstream_url)) {
 		    if ($debug) {
-			warn "$progname warning: Unable to determine upstream url from redirections,\n" .
+			uscan_warn "$progname warning: Unable to determine upstream url from redirections,\n" .
 			    "defaulting to using site specified in watchfile\n";
 		    }
 		    $upstream_url = "$sites[0]$newfile";
@@ -1176,7 +1190,7 @@ EOF
 		}
 		if (!defined($upstream_url)) {
 		    if ($debug) {
-			warn "$progname warning: Unable to determine upstream url from redirections,\n" .
+			uscan_warn "$progname warning: Unable to determine upstream url from redirections,\n" .
 			    "defaulting to using site specified in watchfile\n";
 		    }
 		    $upstream_url = "$urlbase$newfile";
@@ -1191,7 +1205,7 @@ EOF
 	if (exists $options{'downloadurlmangle'}) {
 	    foreach my $pat (@{$options{'downloadurlmangle'}}) {
 		if (! safe_replace(\$upstream_url, $pat)) {
-		    warn "$progname: In $watchfile, potentially"
+		    uscan_warn "$progname: In $watchfile, potentially"
 		      . " unsafe or malformed downloadurlmangle"
 		      . " pattern:\n  '$pat'"
 		      . " found. Skipping watchline\n"
@@ -1206,6 +1220,20 @@ EOF
 	$upstream_url = "$base$newfile";
     }
 
+    if (exists $options{'pgpsigurlmangle'}) {
+	$pgpsig_url = $upstream_url;
+	foreach my $pat (@{$options{'pgpsigurlmangle'}}) {
+	    if (! safe_replace(\$pgpsig_url, $pat)) {
+		uscan_warn "$progname: In $watchfile, potentially"
+		  . " unsafe or malformed pgpsigurlmangle"
+		  . " pattern:\n  '$pat'"
+		  . " found. Skipping watchline\n"
+		  . "  $line\n";
+		return 1;
+	    }
+	}
+    }
+
     $dehs_tags{'debian-uversion'} = $lastversion;
     $dehs_tags{'debian-mangled-uversion'} = $mangled_lastversion;
     $dehs_tags{'upstream-version'} = $newversion;
@@ -1213,7 +1241,7 @@ EOF
 
     # Can't just use $lastversion eq $newversion, as then 0.01 and 0.1
     # compare different, whereas they are treated as equal by dpkg
-    if (system("dpkg", "--compare-versions", "$mangled_lastversion", "eq", "$newversion") == 0) {
+    if (system("dpkg", "--compare-versions", "1:${mangled_lastversion}-0", "eq", "1:${newversion}-0") == 0) {
 	if ($verbose or ($download == 0 and $report and ! $dehs)) {
 	    print $pkg_report_header;
 	    $pkg_report_header = '';
@@ -1240,7 +1268,7 @@ EOF
     # We use dpkg's rules to determine whether our current version
     # is newer or older than the remote version.
     if (!defined $download_version) {
-	if (system("dpkg", "--compare-versions", "$mangled_lastversion", "gt", "$newversion") == 0) {
+	if (system("dpkg", "--compare-versions", "1:${mangled_lastversion}-0", "gt", "1:${newversion}-0") == 0) {
 	    if ($verbose) {
 		print " => remote site does not even have current version\n";
 	    } elsif ($dehs) {
@@ -1313,18 +1341,20 @@ EOF
     # Download newer package
     if ($upstream_url =~ m%^http(s)?://%) {
 	if (defined($1) and !$haveSSL) {
-	    die "$progname: you must have the libcrypt-ssleay-perl package installed\nto use https URLs\n";
+	    uscan_die "$progname: you must have the libcrypt-ssleay-perl package installed\nto use https URLs\n";
 	}
 	# substitute HTML entities
 	# Is anything else than "&amp;" required?  I doubt it.
 	print STDERR "$progname debug: requesting URL $upstream_url\n" if $debug;
-	$request = HTTP::Request->new('GET', $upstream_url);
+	my $headers = HTTP::Headers->new;
+	$headers->header('Accept' => '*/*');
+	$request = HTTP::Request->new('GET', $upstream_url, $headers);
 	$response = $user_agent->request($request, "$destdir/$newfile_base");
 	if (! $response->is_success) {
 	    if (defined $pkg_dir) {
-		warn "$progname warning: In directory $pkg_dir, downloading\n  $upstream_url failed: " . $response->status_line . "\n";
+		uscan_warn "$progname warning: In directory $pkg_dir, downloading\n  $upstream_url failed: " . $response->status_line . "\n";
 	    } else {
-		warn "$progname warning: Downloading\n $upstream_url failed:\n" . $response->status_line . "\n";
+		uscan_warn "$progname warning: Downloading\n $upstream_url failed:\n" . $response->status_line . "\n";
 	    }
 	    return 1;
 	}
@@ -1343,14 +1373,35 @@ EOF
 	}
 	if (! $response->is_success) {
 	    if (defined $pkg_dir) {
-		warn "$progname warning: In directory $pkg_dir, downloading\n  $upstream_url failed: " . $response->status_line . "\n";
+		uscan_warn "$progname warning: In directory $pkg_dir, downloading\n  $upstream_url failed: " . $response->status_line . "\n";
 	    } else {
-		warn "$progname warning: Downloading\n $upstream_url failed:\n" . $response->status_line . "\n";
+		uscan_warn "$progname warning: Downloading\n $upstream_url failed:\n" . $response->status_line . "\n";
 	    }
 	    return 1;
 	}
     }
 
+    if (defined $pgpsig_url) {
+	print "-- Downloading OpenPGP signature for package as $newfile_base.pgp\n" if $verbose;
+	my $sigrequest = HTTP::Request->new('GET', "$pgpsig_url");
+	my $sigresponse = $user_agent->request($sigrequest, "$destdir/$newfile_base.pgp");
+
+	if (! $sigresponse->is_success) {
+	    if (defined $pkg_dir) {
+		uscan_warn "$progname warning: In directory $pkg_dir, downloading OpenPGP signature\n  $upstream_url failed: " . $sigresponse->status_line . "\n";
+	    } else {
+		uscan_warn "$progname warning: Downloading OpenPGP signature\n $pgpsig_url failed:\n" . $sigresponse->status_line . "\n";
+	    }
+	    return 1;
+	}
+
+	print "-- Verifying OpenPGP signature $newfile_base.pgp for $newfile_base\n" if $verbose;
+	system('/usr/bin/gpgv', '--homedir', '/dev/null',
+	       '--keyring', 'debian/upstream-signing-key.pgp',
+	       "$destdir/$newfile_base.pgp", "$destdir/$newfile_base") >> 8 == 0
+		 or uscan_die("$progname warning: OpenPGP signature did not verify.\n");
+    }
+
     if ($repack and $newfile_base =~ /^(.*)\.(tar\.bz2|tbz2?)$/) {
 	print "-- Repacking from bzip2 to gzip\n" if $verbose;
 	my $newfile_base_gz = "$1.tar.gz";
@@ -1400,25 +1451,25 @@ EOF
 	print "-- Repacking from zip to .tar.gz\n" if $verbose;
 
 	system('command -v unzip >/dev/null 2>&1') >> 8 == 0
-	  or die("unzip binary not found. You need to install the package unzip to be able to repack .zip upstream archives.\n");
+	  or uscan_die("unzip binary not found. You need to install the package unzip to be able to repack .zip upstream archives.\n");
 
 	my $newfile_base_gz = "$1.tar.gz";
 	my $tempdir = tempdir ("uscanXXXX", TMPDIR => 1, CLEANUP => 1);
 	# Parent of the target directory should be under our control
 	$tempdir .= '/repack';
-	mkdir $tempdir or die("Unable to mkdir($tempdir): $!\n");
+	mkdir $tempdir or uscan_die("Unable to mkdir($tempdir): $!\n");
 	my $absdestdir = abs_path($destdir);
 	system('unzip', '-q', '-a', '-d', $tempdir, "$destdir/$newfile_base") == 0
-	    or die("Repacking from zip to tar.gz failed (could not unzip)\n");
+	    or uscan_die("Repacking from zip to tar.gz failed (could not unzip)\n");
 	my $cwd = cwd();
-	chdir($tempdir) or die("Unable to chdir($tempdir): $!\n");
+	chdir($tempdir) or uscan_die("Unable to chdir($tempdir): $!\n");
 	eval {
 	    spawn(exec => ['tar', '--owner=root', '--group=root', '--mode=a+rX', '-czf', "$absdestdir/$newfile_base_gz", glob('* .[!.]*')],
 		  env => { GZIP => '-n -9' },
 		  wait_child => 1);
 	};
 	if ($@) {
-	    die("Repacking from zip to tar.gz failed (could not create tarball)\n");
+	    uscan_die("Repacking from zip to tar.gz failed (could not create tarball)\n");
 	}
 	chdir($cwd);
 	unlink "$destdir/$newfile_base";
@@ -1436,7 +1487,7 @@ EOF
 		  wait_child => 1);
 	};
 	unless (defined $filetype && $filetype =~ /compressed data/) {
-	    warn "$progname warning: $destdir/$newfile_base does not appear to be a compressed file;\nthe file command says: $filetype\nNot processing this file any further!\n";
+	    uscan_warn "$progname warning: $destdir/$newfile_base does not appear to be a compressed file;\nthe file command says: $filetype\nNot processing this file any further!\n";
 	    return 1;
 	}
     }
@@ -1564,13 +1615,13 @@ sub newest_dir ($$$$$) {
 
     if ($site =~ m%^http(s)?://%) {
 	if (defined($1) and !$haveSSL) {
-	    die "$progname: you must have the libcrypt-ssleay-perl package installed\nto use https URLs\n";
+	    uscan_die "$progname: you must have the libcrypt-ssleay-perl package installed\nto use https URLs\n";
 	}
 	print STDERR "$progname debug: requesting URL $base\n" if $debug;
 	$request = HTTP::Request->new('GET', $base);
 	$response = $user_agent->request($request);
 	if (! $response->is_success) {
-	    warn "$progname warning: In watchfile $watchfile, reading webpage\n  $base failed: " . $response->status_line . "\n";
+	    uscan_warn "$progname warning: In watchfile $watchfile, reading webpage\n  $base failed: " . $response->status_line . "\n";
 	    return 1;
 	}
 
@@ -1597,7 +1648,7 @@ sub newest_dir ($$$$$) {
 	    }
 	}
 	if (@hrefs) {
-	    @hrefs = Devscripts::Versort::versort(@hrefs);
+	    @hrefs = Devscripts::Versort::upstream_versort(@hrefs);
 	    if ($debug) {
 		print "-- Found the following matching hrefs (newest first):\n";
 		foreach my $href (@hrefs) { print "     $$href[1] ($$href[0])\n"; }
@@ -1608,7 +1659,7 @@ sub newest_dir ($$$$$) {
 	    $newdir =~ s%^.*/%%;
 	    return $newdir;
 	} else {
-	    warn "$progname warning: In $watchfile,\n  no matching hrefs for pattern\n  $site$dir$pattern";
+	    uscan_warn "$progname warning: In $watchfile,\n  no matching hrefs for pattern\n  $site$dir$pattern";
 	    return 1;
 	}
     }
@@ -1629,7 +1680,7 @@ sub newest_dir ($$$$$) {
 	    else { delete $ENV{'FTP_PASSIVE'}; }
 	}
 	if (! $response->is_success) {
-	    warn "$progname warning: In watchfile $watchfile, reading webpage\n  $base failed: " . $response->status_line . "\n";
+	    uscan_warn "$progname warning: In watchfile $watchfile, reading webpage\n  $base failed: " . $response->status_line . "\n";
 	    return '';
 	}
 
@@ -1669,11 +1720,11 @@ sub newest_dir ($$$$$) {
 		print STDERR "-- Found the following matching dirs:\n";
 		foreach my $dir (@dirs) { print STDERR "     $$dir[1]\n"; }
 	    }
-	    @dirs = Devscripts::Versort::versort(@dirs);
+	    @dirs = Devscripts::Versort::upstream_versort(@dirs);
 	    my ($newversion, $newdir) = @{$dirs[0]};
 	    return $newdir;
 	} else {
-	    warn "$progname warning: In $watchfile no matching dirs for pattern\n  $base$pattern\n";
+	    uscan_warn "$progname warning: In $watchfile no matching dirs for pattern\n  $base$pattern\n";
 	    return '';
 	}
     }
@@ -1689,7 +1740,7 @@ sub process_watchfile ($$$$)
     %dehs_tags = ();
 
     unless (open WATCH, $watchfile) {
-	warn "$progname warning: could not open $watchfile: $!\n";
+	uscan_warn "$progname warning: could not open $watchfile: $!\n";
 	return 1;
     }
 
@@ -1702,7 +1753,7 @@ sub process_watchfile ($$$$)
 	chomp;
 	if (s/(?<!\\)\\$//) {
 	    if (eof(WATCH)) {
-		warn "$progname warning: $watchfile ended with \\; skipping last line\n";
+		uscan_warn "$progname warning: $watchfile ended with \\; skipping last line\n";
 		$status=1;
 		last;
 	    }
@@ -1715,12 +1766,12 @@ sub process_watchfile ($$$$)
 		$watch_version=$1;
 		if ($watch_version < 2 or
 		    $watch_version > $CURRENT_WATCHFILE_VERSION) {
-		    warn "$progname ERROR: $watchfile version number is unrecognised; skipping watchfile\n";
+		    uscan_warn "$progname ERROR: $watchfile version number is unrecognised; skipping watchfile\n";
 		    last;
 		}
 		next;
 	    } else {
-		warn "$progname warning: $watchfile is an obsolete version 1 watchfile;\n  please upgrade to a higher version\n  (see uscan(1) for details).\n";
+		uscan_warn "$progname warning: $watchfile is an obsolete version 1 watchfile;\n  please upgrade to a higher version\n  (see uscan(1) for details).\n";
 		$watch_version=1;
 	    }
 	}
@@ -1743,7 +1794,7 @@ sub process_watchfile ($$$$)
     }
 
     close WATCH or
-	$status=1, warn "$progname warning: problems reading $watchfile: $!\n";
+	$status=1, uscan_warn "$progname warning: problems reading $watchfile: $!\n";
 
     return $status;
 }
@@ -1757,21 +1808,31 @@ sub dehs_msg ($)
     push @{$dehs_tags{'messages'}}, $msg;
 }
 
-sub dehs_warn ($)
+sub uscan_warn (@)
 {
-    my $warning = $_[0];
-    $warning =~ s/\s*$//;
-    push @{$dehs_tags{'warnings'}}, $warning;
+    if ($dehs) {
+	my $warning = $_[0];
+	$warning =~ s/\s*$//;
+	push @{$dehs_tags{'warnings'}}, $warning;
+    }
+    else {
+	warn @_;
+    }
 }
 
-sub dehs_die ($)
+sub uscan_die (@)
 {
-    my $msg = $_[0];
-    $msg =~ s/\s*$//;
-    %dehs_tags = ('errors' => "$msg");
-    $dehs_end_output=1;
-    dehs_output;
-    exit 1;
+    if ($dehs) {
+	my $msg = $_[0];
+	$msg =~ s/\s*$//;
+	%dehs_tags = ('errors' => "$msg");
+	$dehs_end_output=1;
+	dehs_output;
+	exit 1;
+    }
+    else {
+	die @_;
+    }
 }
 
 sub dehs_output ()
